@@ -95,9 +95,11 @@ class Canvas(QtWidgets.QWidget):
         self.penColor = QtCore.Qt.black
         self.immersive = False
         self.immersive_first = True
+        self.immersive_ok = False
         self.keyboard_button = pynput.keyboard.Controller()
         self.keyboard_key = pynput.keyboard.Key
 
+
         fn = str(osp.join(osp.dirname(osp.realpath(__file__)), 'crossSave.txt'))
         # Load previous cross-line style
         if osp.isfile(fn) and self.firstLoad == False:
@@ -180,7 +182,7 @@ class Canvas(QtWidgets.QWidget):
         self.selectedShapes = []
         for shape in self.shapes:
             shape.selected = False
-        self.repaint()
+        self.update()
 
     def enterEvent(self, ev):
         self.overrideCursor(self._cursor)
@@ -270,7 +272,7 @@ class Canvas(QtWidgets.QWidget):
             elif self.createMode == 'point':
                 self.line.points = [self.current[0]]
                 self.line.close()
-            self.repaint()
+            self.update()
             self.current.highlightClear()
             return
         
@@ -285,23 +287,23 @@ class Canvas(QtWidgets.QWidget):
             if self.selectedShapesCopy and self.prevPoint:
                 self.overrideCursor(CURSOR_MOVE)
                 self.boundedMoveShapes(self.selectedShapesCopy, pos)
-                self.repaint()
+                self.update()
             elif self.selectedShapes:
                 self.selectedShapesCopy = \
                     [s.copy() for s in self.selectedShapes]
-                self.repaint()
+                self.update()
             return
 
         # Polygon/Vertex moving.
         if QtCore.Qt.LeftButton & ev.buttons():
             if self.selectedVertex():
                 self.boundedMoveVertex(pos)
-                self.repaint()
+                self.update()
                 self.movingShape = True
             elif self.selectedShapes and self.prevPoint:
                 self.overrideCursor(CURSOR_MOVE)
                 self.boundedMoveShapes(self.selectedShapes, pos)
-                self.repaint()
+                self.update()
                 self.movingShape = True
             return
 
@@ -378,6 +380,7 @@ class Canvas(QtWidgets.QWidget):
             pos = self.transformPos(ev.posF())
         if ev.button() == QtCore.Qt.LeftButton:
             if self.drawing():
+                self.immersive_ok = True
                 if self.immersive and self.immersive_first:
                     self.keyboard_con()
                     self.immersive_first = False
@@ -414,12 +417,12 @@ class Canvas(QtWidgets.QWidget):
                 group_mode = (int(ev.modifiers()) == QtCore.Qt.ControlModifier)
                 self.selectShapePoint(pos, multiple_selection_mode=group_mode)
                 self.prevPoint = pos
-                self.repaint()
+                self.update()
         elif ev.button() == QtCore.Qt.RightButton and self.editing():
             group_mode = (int(ev.modifiers()) == QtCore.Qt.ControlModifier)
             self.selectShapePoint(pos, multiple_selection_mode=group_mode)
             self.prevPoint = pos
-            self.repaint()
+            self.update()
 
     def mouseReleaseEvent(self, ev):
         if ev.button() == QtCore.Qt.RightButton:
@@ -429,7 +432,7 @@ class Canvas(QtWidgets.QWidget):
                     and self.selectedShapesCopy:
                 # Cancel the move by deleting the shadow copy.
                 self.selectedShapesCopy = []
-                self.repaint()
+                self.update()
         elif ev.button() == QtCore.Qt.LeftButton and self.selectedShapes:
             self.overrideCursor(CURSOR_GRAB)
 
@@ -454,7 +457,7 @@ class Canvas(QtWidgets.QWidget):
             for i, shape in enumerate(self.selectedShapesCopy):
                 self.selectedShapes[i].points = shape.points
         self.selectedShapesCopy = []
-        self.repaint()
+        self.update()
         self.storeShapes()
         return True
 
@@ -464,7 +467,7 @@ class Canvas(QtWidgets.QWidget):
             # Only hide other shapes if there is a current selection.
             # Otherwise the user will not be able to select a shape.
             self.setHiding(True)
-            self.repaint()
+            self.update()
 
     def setHiding(self, enable=True):
         self._hideBackround = self.hideBackround if enable else False
@@ -756,8 +759,9 @@ class Canvas(QtWidgets.QWidget):
     def keyPressEvent(self, ev):
         key = ev.key()
         if key == QtCore.Qt.Key_Escape and self.current:
-            self.keyboard_con()
-            self.immersive_first = True
+            if self.immersive:
+                self.keyboard_con()
+                self.immersive_first = True
             self.current = None
             self.drawingPolygon.emit(False)
             self.update()
@@ -793,13 +797,13 @@ class Canvas(QtWidgets.QWidget):
         else:
             self.current = None
             self.drawingPolygon.emit(False)
-        self.repaint()
+        self.update()
 
     def loadPixmap(self, pixmap, clear_shapes=True):
         self.pixmap = pixmap
         if clear_shapes:
             self.shapes = []
-        self.repaint()
+        self.update()
         
     def loadShapes(self, shapes, replace=True):
         if replace:
@@ -811,11 +815,11 @@ class Canvas(QtWidgets.QWidget):
         self.hShape = None
         self.hVertex = None
         self.hEdge = None
-        self.repaint()
+        self.update()
 
     def setShapeVisible(self, shape, value):
         self.visible[shape] = value
-        self.repaint()
+        self.update()
 
     def overrideCursor(self, cursor):
         self.restoreCursor()
